precision mediump float;

uniform lowp float decay;
uniform lowp float exposure;
uniform lowp float density;
uniform lowp float weight;
uniform lowp float light0X;
uniform lowp float light0Y;

/////////////////////////////////////////////////////////
// GodRays

//The current foreground texture co-ordinate
varying mediump vec2 vTex;
//The foreground texture sampler, to be sampled at vTex
uniform lowp sampler2D samplerFront;
//The current foreground rectangle being rendered
uniform mediump vec2 srcStart;
uniform mediump vec2 srcEnd;
//The current foreground source rectangle being rendered
uniform mediump vec2 srcOriginStart;
uniform mediump vec2 srcOriginEnd;
//The current foreground source rectangle being rendered, in layout 
uniform mediump vec2 layoutStart;
uniform mediump vec2 layoutEnd;
//The background texture sampler used for background - blending effects
uniform lowp sampler2D samplerBack;
//The current background rectangle being rendered to, in texture co-ordinates, for background-blending effects
uniform mediump vec2 destStart;
uniform mediump vec2 destEnd;
//The time in seconds since the runtime started. This can be used for animated effects
uniform mediump float seconds;
//The size of a texel in the foreground texture in texture co-ordinates
uniform mediump vec2 pixelSize;
//The current layer scale as a factor (i.e. 1 is unscaled)
uniform mediump float layerScale;
//The current layer angle in radians.
uniform mediump float layerAngle;

#define NUM_SAMPLES 100

void main()
{
    vec2 tc = (vTex-srcOriginStart)/(srcOriginEnd-srcOriginStart);
    vec2 n = tc;

    vec2 deltaTexCoord = tc - vec2(light0X, light0Y);
    deltaTexCoord *= 1.0 / float(NUM_SAMPLES) * density;
    
    float illuminationDecay = 1.0;
    vec4 color =texture2D(samplerFront, vTex)*0.4;
    for(int i=0; i < NUM_SAMPLES ; i++)
    {
       tc -= deltaTexCoord;
       vec4 sample = texture2D(samplerFront, mix(srcOriginStart, srcOriginEnd, tc))*0.4;
       sample *= illuminationDecay * weight;
       color += sample;
       illuminationDecay *= decay;
    }
    vec4 realColor = texture2D(samplerBack, mix(destStart, destEnd, n));
    // gl_FragColor = color + realColor;
    /// This will "smooth" the rays and render more realistic
    gl_FragColor = ((vec4((vec3(color.r, color.g, color.b) * exposure), 1)) + (realColor*(1.1)));
}